VERSION 5.00
Begin VB.UserControl DiskData 
   AutoRedraw      =   -1  'True
   BackStyle       =   0  'Transparent
   CanGetFocus     =   0   'False
   ClientHeight    =   900
   ClientLeft      =   0
   ClientTop       =   0
   ClientWidth     =   1095
   InvisibleAtRuntime=   -1  'True
   PropertyPages   =   "DiskData.ctx":0000
   ScaleHeight     =   900
   ScaleWidth      =   1095
   ToolboxBitmap   =   "DiskData.ctx":0024
   Begin VB.Image Image1 
      Appearance      =   0  'Flat
      Height          =   480
      Left            =   120
      Picture         =   "DiskData.ctx":0336
      Top             =   120
      Width           =   480
   End
End
Attribute VB_Name = "DiskData"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

'All the API calls...
'This one gets all the information needed about a drive.
Private Declare Function GetVolumeInformation Lib "kernel32" Alias _
    "GetVolumeInformationA" (ByVal lpRootPathName As String, _
    ByVal lpVolumeNameBuffer As String, ByVal nVolumeNameSize As Long, _
    lpVolumeSerialNumber As Long, lpMaximumComponentLength As Long, _
    lpFileSystemFlags As Long, ByVal lpFileSystemNameBuffer As String, _
    ByVal nFileSystemNameSize As Long) As Long
    
'This one returns the free space on a drive.
Private Declare Function GetDiskFreeSpace Lib "kernel32" _
   Alias "GetDiskFreeSpaceA" (ByVal lpRootPathName As String, _
   lpSectorsPerCluster As Long, lpBytesPerSector As Long, _
   lpNumberOfFreeClusters As Long, _
   lpTtoalNumberOfClusters As Long) As Long
   
'This one gets the type of drive, which is an integer. We simplify the result by
'using the ddDriveConstants enumeration.
Private Declare Function GetDriveType Lib "kernel32" _
   Alias "GetDriveTypeA" (ByVal nDrive As String) As Long

'This is the one I was looking for; an API call to change the drive label. Of
'course, if the Drive Properties dialog could do it, so could VB. Somebody
'asked me if it would be possible to change the serial number of a drive, and
'I think that it would be very unwise. A quick look in the API viewer proved
'this.
Private Declare Function SetVolumeLabel Lib "kernel32" _
    Alias "SetVolumeLabelA" (ByVal lpRootPathName As String, _
    ByVal lpVolumeName As String) As Long

'This one takes an ASCII code and tells you whether or not the related
'character is a letter.
Private Declare Function IsCharAlpha Lib "user32" Alias "IsCharAlphaA" _
    (ByVal cChar As Byte) As Long

'Finally this one lets you format a disk.
Private Declare Function SHFormatDrive Lib "shell32" (ByVal hWnd As Long, _
    ByVal Drive As Long, ByVal fmtID As Long, ByVal Options As Long) As Long
          
'Default Property Values
Const m_def_Drive = "" 'No drive is selected!

'Property Variables:
Dim m_DriveFullSpace As Long
Dim m_DriveLabel As String
Dim m_DriveSerial As String
Dim m_DriveType As ddDriveConstants
Dim m_DriveFreeSpace As Long
Dim m_Drive As String

'Enumeration
Enum ddDriveConstants
   ddMissing = 1            'Drive doesn't exist
   ddRemovable = 2    'eg. a floppy disk
   ddFixed = 3               'a hard disk
   ddRemoteDrive = 4  'a network drive
   ddCDRom = 5          'guess what?
   ddRAMDisk = 6          'a RAM disk
End Enum

Enum eOptions
    FORMAT_QUICK = 0
    FORMAT_FULL = 1
    FORMAT_SYSFILESONLY = 2
End Enum

'Default Property Values:
Const m_def_DriveFullSpace = 0
Const m_def_DriveLabel = "0"
Const m_def_DriveSerial = "0"
Const m_def_DriveType = 0
Const m_def_DriveFreeSpace = 0

Public Sub GetVolumeInfo(DriveLetter As String)
    'Change the drive we want to get information about.
    
    'This requires a single character for the drive name. If we have anything
    'other than this, it's an error.
    If Len(DriveLetter) <> 1 Or IsCharAlpha(Asc(DriveLetter)) = False Then
        Err.Raise 8001, , "Drive property must be a single alphabetic character."
    End If
    m_Drive = DriveLetter
    PropertyChanged "Drive"
    
    'First we get the drive label and serial number.
    Dim r&, PathName$, DrvVolumeName$, DrvSerialNo$
       
    PathName$ = Drive & ":\"
    rgbGetVolumeInformationRDI PathName$, DrvVolumeName$, _
    DrvSerialNo$
     
    m_DriveLabel = DrvVolumeName$
    m_DriveSerial = DrvSerialNo$
    
    'Then we get the drive type...
    m_DriveType = GetDriveType(DriveLetter & ":\")
    
    'Now the hard part; the drive's free space and the capacity of the drive.
    Dim SecPerCluster As Long 'sectors per cluster
    Dim BytesPerSector As Long
    Dim FreeClusters As Long: Dim TotalClusters As Long
    
    GetDiskFreeSpace DriveLetter & ":\", SecPerCluster, BytesPerSector, _
        FreeClusters, TotalClusters
    
    m_DriveFreeSpace = SecPerCluster * BytesPerSector * FreeClusters
    m_DriveFullSpace = SecPerCluster * BytesPerSector * TotalClusters
    
End Sub

Private Sub rgbGetVolumeInformationRDI(PathName$, DrvVolumeName$, DrvSerialNo$)
    
    Dim r&, pos%
    Dim HiWord&, HiHexStr$, LoWord&, LoHexStr$
    Dim VolumeSN&, MaxFNLen&
    Dim dummyStr$, dummyVal1&, dummyVal2&
    '--------------------------------------------
    'create some working strings
    DrvVolumeName$ = Space$(14)
    dummyStr$ = Space$(32)
    '--------------------------------------------
    'do what it says
    r& = GetVolumeInformation(PathName$, _
    DrvVolumeName$, _
    Len(DrvVolumeName$), _
    VolumeSN&, dummyVal1&, _
    dummyVal2&, _
    dummyStr$, _
    Len(dummyStr$))
    'error check
    If r& = 0 Then Exit Sub
    '--------------------------------------------
    'the volume label
    pos% = InStr(DrvVolumeName$, Chr$(0))
    If pos% Then DrvVolumeName$ = Left$(DrvVolumeName$, pos% - 1)
    If Len(Trim$(DrvVolumeName$)) = 0 Then DrvVolumeName$ = "(no label)"
    
    '--------------------------------------------
    'finally, determine the drive volume id
    HiWord& = GetHiWord(VolumeSN&) And &HFFFF&
    LoWord& = GetLoWord(VolumeSN&) And &HFFFF&
    Rem*************************************************
    Rem Note:fix by Barry Davies (Barry@Guestline.com)
    Rem next two original lines are commented out
    'HiHexStr$ = Format$(Hex(HiWord&), "0000")
    'LoHexStr$ = Format$(Hex(LoWord&), "0000")
    
    HiHexStr$ = Left$(Hex(HiWord&), 4)
    If Len(HiHexStr$) < 4 Then HiHexStr$ = HiHexStr$ & String(4 - Len(HiHexStr$), "A")
    LoHexStr$ = Left$(Hex(LoWord&), 4)
    If Len(LoHexStr$) < 4 Then LoHexStr$ = LoHexStr$ & String(4 - Len(LoHexStr$), "A")
    Rem end of fix**************************************
    DrvSerialNo$ = HiHexStr$ & "-" & LoHexStr$
End Sub

Private Function GetHiWord(dw As Long) As Integer

   If dw& And &H80000000 Then
      GetHiWord% = (dw& \ 65535) - 1
   Else
      GetHiWord% = dw& \ 65535
   End If
    
End Function

Private Function GetLoWord(dw As Long) As Integer
    
   If dw& And &H8000& Then
     GetLoWord% = &H8000 Or (dw& And &H7FFF&)
   Else
     GetLoWord% = dw& And &HFFFF&
   End If

End Function

Private Sub UserControl_Resize()
   With Image1
      .Left = 0
      .Top = 0
      Width = .Width
      Height = .Height
   End With
   UserControl.Refresh
End Sub

'Initialize Properties for User Control
Private Sub UserControl_InitProperties()
   m_Drive = m_def_Drive
    m_DriveFullSpace = m_def_DriveFullSpace
    m_DriveLabel = m_def_DriveLabel
    m_DriveSerial = m_def_DriveSerial
    m_DriveType = m_def_DriveType
    m_DriveFreeSpace = m_def_DriveFreeSpace
End Sub

'Load property values from storage
Private Sub UserControl_ReadProperties(PropBag As PropertyBag)
   m_Drive = PropBag.ReadProperty("Drive", m_def_Drive)
    m_DriveFullSpace = PropBag.ReadProperty("DriveFullSpace", m_def_DriveFullSpace)
    m_DriveLabel = PropBag.ReadProperty("DriveLabel", m_def_DriveLabel)
    m_DriveSerial = PropBag.ReadProperty("DriveSerial", m_def_DriveSerial)
    m_DriveType = PropBag.ReadProperty("DriveType", m_def_DriveType)
    m_DriveFreeSpace = PropBag.ReadProperty("DriveFreeSpace", m_def_DriveFreeSpace)
End Sub

'Write property values to storage
Private Sub UserControl_WriteProperties(PropBag As PropertyBag)
   Call PropBag.WriteProperty("Drive", m_Drive, m_def_Drive)
    Call PropBag.WriteProperty("DriveFullSpace", m_DriveFullSpace, m_def_DriveFullSpace)
    Call PropBag.WriteProperty("DriveLabel", m_DriveLabel, m_def_DriveLabel)
    Call PropBag.WriteProperty("DriveSerial", m_DriveSerial, m_def_DriveSerial)
    Call PropBag.WriteProperty("DriveType", m_DriveType, m_def_DriveType)
    Call PropBag.WriteProperty("DriveFreeSpace", m_DriveFreeSpace, m_def_DriveFreeSpace)
End Sub
Public Property Get Drive() As String
Attribute Drive.VB_Description = "Returns or sets the drive letter to get information for. MUST be a single character."
Attribute Drive.VB_MemberFlags = "400"
   Drive = m_Drive
End Property

Public Property Get DriveFullSpace() As Long
Attribute DriveFullSpace.VB_Description = "Returns the total capacity of the drive in bytes."
    DriveFullSpace = m_DriveFullSpace
End Property

Public Property Get DriveLabel() As String
Attribute DriveLabel.VB_Description = "Returns/sets the drive label."
Attribute DriveLabel.VB_MemberFlags = "400"
    DriveLabel = m_DriveLabel
End Property

Public Property Let DriveLabel(ByVal New_DriveLabel As String)
    'The exciting part; we get to change the drive label!
    If Len(New_DriveLabel) > 11 Then
        Err.Raise 8003, , _
            "The new drive label cannot be more than 11 characters long."
    End If
    
    'After a bit of research, there are some characters the drive label will not
    'allow.
    If InStr(1, New_DriveLabel, Chr(34)) Or InStr(1, New_DriveLabel, "*") Or _
        InStr(1, New_DriveLabel, "?") Then
        MsgBox "The drive label cannot contain the following characters: " & _
            Chr(34) & " * ?", vbExclamation
        Exit Property
    End If
    
    If SetVolumeLabel(m_Drive & ":\", New_DriveLabel) Then
        m_DriveLabel = New_DriveLabel
        PropertyChanged "DriveLabel"
    End If
End Property

Public Property Get DriveSerial() As String
Attribute DriveSerial.VB_Description = "Returns the serial number of the drive."
Attribute DriveSerial.VB_MemberFlags = "400"
    DriveSerial = m_DriveSerial
End Property

Public Property Get DriveType() As ddDriveConstants
Attribute DriveType.VB_Description = "Returns the type of drive."
Attribute DriveType.VB_MemberFlags = "400"
    DriveType = m_DriveType
End Property

Public Property Get DriveFreeSpace() As Long
Attribute DriveFreeSpace.VB_Description = "Returns the amount of free space on the drive in bytes."
    DriveFreeSpace = m_DriveFreeSpace
End Property

Public Function FormatDrive(DriveLetter As String, lOptions As eOptions) _
    As Boolean
    Const SHFMT_ID_DEFAULT = &HFFFF
    
    FormatDrive = SHFormatDrive(UserControl.Parent.hWnd, _
        (Asc(DriveLetter) - 65), SHFMT_ID_DEFAULT, lOptions)
End Function

Sub ShowAbout()
Attribute ShowAbout.VB_UserMemId = -552
Attribute ShowAbout.VB_MemberFlags = "40"
    frmAbout.Show vbModal
End Sub
